/*
 * this routine is based on:
 * 
 * appl/sample/sclient/sclient.c
 *
 * Sample Kerberos v5 client.
 *
 */

#include "krb5.h"
#include "com_err.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#else
extern char *malloc();
#endif

#include <pwd.h>

#include "sample.h"

int mhd( char *user){

    struct servent *sp;
    struct hostent *hp;
    struct sockaddr_in sin, lsin;
    int sock, namelen;
    krb5_context context;
    krb5_data recv_data;
    krb5_data cksum_data;
    krb5_error_code retval;
    krb5_ccache ccdef;
    krb5_principal client, server;
    krb5_error *err_ret;
    krb5_ap_rep_enc_part *rep_ret;
    krb5_auth_context auth_context = 0;
    short xmitlen;
    char *service = SAMPLE_SERVICE;
    FILE *f;
    char buf[1024];
    struct passwd *pwd;
    uid_t uid;
    char *nfshost;

    /* first of all, read our NFS server name from file */

    f = fopen("/usr/local/etc/homeserver","r");
    if ( f == (FILE *) NULL ) {
	perror("error opening homeserver file");
	return(-1);
    }

    buf[0]='\0';
    while ( ( ! feof(f) ) && (fgets( buf, 1024, f) ) ) {
      if ( ( strlen(buf) > 1 ) && ( buf[0] != '#' )) {
	buf[strlen(buf)-1] = '\0';
	break;
      }
    }
    fclose(f);
    if ( strlen(buf) < 1 ) {
	fprintf(stderr, "did not find a Home Directory Server name in /usr/local/etc/homeserver\n");
	return(-1);
    }

    nfshost = (char *) malloc( strlen(buf)+2 );
    strcpy(nfshost, buf);

    fprintf(stderr, "sending Home Dir Mount request to %s\n", nfshost);

    /* we need to convert username to uid in order to set up KRB5CCNAME */
    if ( (pwd = getpwnam(user)) == (struct passwd *) 0 ) {
	perror("MountHomeDir user lookup error");
	return(-1);
    }
    uid = pwd->pw_uid;
/* kml 200006302004 don't need these bits the krb5 pam mod does it */
/*    sprintf(buf,"/tmp/krb5cc_%d",uid); */

    /* actually I'm sure there is a way to do this directly. */
    /* will find it and fix this r.s.n. */
/*    if ( setenv("KRB5CCNAME", buf, 1) ){
	perror("MountHomeDir setenv error");
	return(-1);
    } */


    retval = krb5_init_context(&context);
    if (retval) {
	    com_err("MountHomeDir error", retval, "while initializing krb5");
	    return(-1);
    }

    (void) signal(SIGPIPE, SIG_IGN);
    if (!valid_cksumtype(CKSUMTYPE_CRC32)) {
	com_err("MountHomeDir error", KRB5_PROG_SUMTYPE_NOSUPP, "while using CRC-32");
	return(-1);
    }

    /* clear out the structure first */
    (void) memset((char *)&sin, 0, sizeof(sin));

    /* find the port number for knetd */
    sp = getservbyname(SAMPLE_PORT, "tcp");
    if (!sp) {
      fprintf(stderr,
	    "unknown service %s/tcp; check /etc/services\n",
	    SAMPLE_PORT);
      return(-1);
    }
    /* copy the port number */
    sin.sin_port = sp->s_port;
    sin.sin_family = AF_INET;
    
    /* look up the server host */
    hp = gethostbyname(nfshost);
    if (!hp) {
	fprintf(stderr, "unknown host %s\n",nfshost);
	return(-1);
    }

    if (retval = krb5_sname_to_principal(context, nfshost, service,
					 KRB5_NT_SRV_HST, &server)) {
	com_err("MountHomeDir error", retval, "while creating server name for %s/%s",
		nfshost, service);
	return(-1);
    }

    /* set up the address of the foreign socket for connect() */
    sin.sin_family = hp->h_addrtype;
    (void) memcpy((char *)&sin.sin_addr,
		  (char *)hp->h_addr,
		  sizeof(hp->h_addr));

    /* open a TCP socket */
    sock = socket(PF_INET, SOCK_STREAM, 0);
    if (sock < 0) {
	perror("socket");
	return(-1);
    }

    /* connect to the server */
    if (connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
	perror("connect");
	close(sock);
	return(-1);
    }

    /* find out who I am, now that we are connected and therefore bound */
    namelen = sizeof(lsin);
    if (getsockname(sock, (struct sockaddr *) &lsin, &namelen) < 0) {
	perror("getsockname");
	close(sock);
	return(-1);
    }

    cksum_data.data = nfshost;
    cksum_data.length = strlen(nfshost);

    if (retval = krb5_cc_default(context, &ccdef)) {
	com_err("MountHomeDir error", retval, "while getting default ccache");
	return(-1);
    }

    if (retval = krb5_cc_get_principal(context, ccdef, &client)) {
	com_err("MountHomeDir error", retval, "while getting client principal name");
	return(-1);
    }
    retval = krb5_sendauth(context, &auth_context, (krb5_pointer) &sock,
			   SAMPLE_VERSION, client, server,
			   AP_OPTS_MUTUAL_REQUIRED,
			   &cksum_data,
			   0,		/* no creds, use ccache instead */
			   ccdef, &err_ret, &rep_ret, NULL);

    krb5_free_principal(context, server);	/* finished using it */
    krb5_free_principal(context, client);      
    krb5_cc_close(context, ccdef);
    if ( auth_context != (krb5_auth_context) 0 ) {
      krb5_auth_con_free(context, auth_context);
    }
    if (retval && retval != KRB5_SENDAUTH_REJECTED) {
	com_err("MountHomeDir error", retval, " (caused sendauth failure)");
	return(-1);
    }
    if (retval == KRB5_SENDAUTH_REJECTED) {
	/* got an error */
	printf("sendauth rejected, error reply is:\n\t\"%*s\"\n",
	       err_ret->text.length, err_ret->text.data);
    } else if (rep_ret) {
	/* got a reply */
	krb5_free_ap_rep_enc_part(context, rep_ret);

/*	printf("sendauth succeeded, reply is:\n"); */
	if ((retval = krb5_net_read(context, sock, (char *)&xmitlen,
				    sizeof(xmitlen))) <= 0) {
	    if (retval == 0)
		errno = ECONNABORTED;
	    com_err("MountHomeDir error", errno, "while reading data from server");
	    return(-1);
	}
	recv_data.length = ntohs(xmitlen);
	if (!(recv_data.data = (char *)malloc(recv_data.length + 1))) {
	    com_err("MountHomeDir error", ENOMEM,
		    "while allocating buffer to read from server");
	    return(-1);
	}
	if ((retval = krb5_net_read(context, sock, (char *)recv_data.data,
				    recv_data.length)) <= 0) {
	    if (retval == 0)
		errno = ECONNABORTED;
	    com_err("MountHomeDir error", errno, "while reading data from server");
	    return(-1);
	}
	recv_data.data[recv_data.length] = '\0';
	printf("Home Dir reply received, contents:\n%s\n", recv_data.data);

	if ( worker(recv_data.data) ) {
		free(recv_data.data);
		krb5_free_context(context);
		return(-1);
	}

	free(recv_data.data);
    } else {
	com_err("MountHomeDir error", 0, "no error or reply from sendauth!");
	return(-1);
    }
    krb5_free_context(context);
    return(0);
}
