
/*
 * start_seti.c
 * 
 *   if seti not already running, start it
 *
 */

#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <syslog.h>
#include <string.h>
#include <netdb.h>
extern int h_errno;

void start_seti() {

  /* 
   * if seti is running because we started it, 
   * we should have a /var/run/seti.pid
   */
  
  int i;
  int pf, pe, len;	/* pidfile, proc entry */
  struct stat s;
  char buf[1024];
  pid_t pid;
  FILE *f, *fo, *fi;
  int found;
  char setistr[] = "duey.cc.monash.edu.au:/seti /seti nfs rw 0 0\n";
  char *p;
  struct hostent *h;
  char myname[128];
  char myip[128];

  pf = open("/var/run/seti.pid",O_RDONLY);
  if ( ( pf < 0 ) && ( errno != ENOENT ) ) {
    syslog( LOG_ERR, "start_seti can't open pidfile %m");
    return;
  }

  if ( pf >=0 ) {
    len = read( pf, buf, 1024);
    close(pf);
    if ( len < 0 ) {
      syslog( LOG_ERR, "start_seti can't read pidfile %m");
      return;
    }

    if ( len > 0 ){
      buf[len] = '\0';
      if ( ! sscanf( buf, "%d", &pid ) ) {
	syslog( LOG_ERR, "start_seti can't find pid in pidfile");
	return;
      }
      snprintf(buf,1024,"/proc/%d/cmdline",pid);
      pe = open(buf,O_RDONLY);
      if ( ( pe < 0 ) && ( errno != ENOENT ) ) {
        syslog( LOG_ERR, "start_seti can't open procfile %m");
        return;
      }
      if ( pe >= 0 ) {
	len = read( pe, buf, 1024);
  	close(pe);
	if ( len < 0 ) {
          syslog( LOG_ERR, "start_seti can't read cmdline file %m");
	  return;
	}
        if ( len > 0 ){
	  buf[len] = '\0';
	  if ( strstr( buf, "seti" ) ) return; /* no action needed */
          syslog( LOG_ERR, "start_seti can't find seti in cmdline file %m");
	  /*  return; */
	}
      }
    }
  }

  /* ok, it isn't already running afaict */

  /* make the directory */

  if ( mkdir( "/seti", 0755 ) && ( errno != EEXIST) ) {
    syslog(LOG_ERR,"start_seti - SETI Mountpoint Creation failed");
    return;
  }

  /* mount duey:/seti */

  f = fopen("/etc/fstab","r+");
  if ( f == (FILE *) NULL ) {
    syslog(LOG_ERR,"start_seti - error opening fstab");
    return;
  }
  found = 0;
  while ( ( ! feof(f) ) && (fgets( buf, 1024, f) ) ) {
    if ( strstr(buf,setistr) ) {found = 1; break;}
  }
  if ( ! found ) {
    if ( fputs(setistr,f) < 0 ){
      fclose(f);
      syslog(LOG_ERR,"start_seti - error writing fstab");
      return;
    }
  }
  fclose(f);
  f = fopen("/etc/mtab","r");
  if ( f == (FILE *) NULL ) {
    syslog(LOG_ERR,"start_seti - opening mtab");
    return;
  }
  found = 0;
  /* compare on setistr up to first space */
  p = index(setistr,' ');
  *p = '\0';
  while ( ( ! feof(f) ) && (fgets( buf, 1024, f) ) ) {
    if ( strstr(buf,setistr) ) {found = 1; break;}
  }
  fclose(f);
  if ( ! found ) {
    sprintf(buf,"mount %s",setistr);
    i = system(buf);
    if ( i ) {
      syslog(LOG_ERR,"start_seti - mount failed");
      return;
    }
  }

  /* its mounted, now make our dir - use ip address */
  if ( gethostname(myname,128) ){
    syslog(LOG_ERR,"start_seti - getting hostname");
    return;
  }

  if ( ! (h = gethostbyname( myname )) ) {
    syslog(LOG_ERR,"start_seti - getting hostip");
    return;
  }
  sprintf(myip,"%u.%u.%u.%u",(unsigned char) h->h_addr_list[0][0]
			,(unsigned char) h->h_addr_list[0][1]
			,(unsigned char) h->h_addr_list[0][2]
			,(unsigned char) h->h_addr_list[0][3] );
  sprintf(buf,"/seti/%s",myip);
  
  /* become keithl since duey doesnt want to export rw with root access */
  if ( seteuid( 703) ){
    syslog(LOG_ERR,"start_seti - seteuid failed %m");
    return;
  }

  if ( mkdir( buf, 0755 ) && ( errno != EEXIST) ) {
    syslog(LOG_ERR,"start_seti - SETI Dir Creation failed %m");
    return;
  }

  /* cd into it */
  if ( chdir( buf ) ){
    syslog(LOG_ERR,"start_seti - chdir failed %m");
    return;
  }

  /* if we don't have a user_info.sah copy from .. */
  if (  stat("user_info.sah", &s) ){
    if ( errno != ENOENT ) {
      syslog(LOG_ERR,"start_seti - stat user_info.sah failed %m");
      return;
    }
    fi = fopen("../user_info.sah","r");
    if ( fi == (FILE *) NULL ) {
      syslog(LOG_ERR,"start_seti - error opening user_info for reading %m");
      return;
    }
    fo = fopen("user_info.sah","w");
    if ( fo == (FILE *) NULL ) {
      syslog(LOG_ERR,"start_seti - error opening user_info for writing %m");
      return;
    }
    while ( ( ! feof(f) ) && (fgets( buf, 1024, fi) ) ) {
      if ( fputs(buf,fo) < 0 ){
	fclose(fi);
	fclose(fo);
	syslog(LOG_ERR,"start_seti - error writing user_info %m");
	return;
      }
    }
  }
  
  /* now there isn't much left to do execpet start it, and write the pid */

  pid = fork();

  if ( pid == (pid_t) -1 ){
    syslog(LOG_ERR,"start_seti - cant fork %m");
    return;
  }
  if ( pid == (pid_t) 0 ) {
    close(0);
    close(1);
    close(2);
    if ( execl( "/seti/setiathome", 
         "seti",
         "-email", 
         "-nice", 
         "19", 
         "-proxy", 
         "proxy",
         0) ){
      syslog(LOG_ERR,"start_seti - cant execl %m");
      return;
    }
  }
  if ( pid > (pid_t) 0 ) {

    /* become root again to write the pid file */
    if ( seteuid( 0 ) ){
      syslog(LOG_ERR,"start_seti - seteuid root failed %m");
      return;
    }

    /* cd out since we may wish to unmount this */

    if ( chdir( "/root" ) ){
      syslog(LOG_ERR,"start_seti - chdir /root failed %m");
      return;
    }

    f = fopen("/var/run/seti.pid","w");
    if ( f == (FILE *) NULL ) {
      syslog(LOG_ERR,"start_seti - cant open pid file for writing  %m");
      return;    
    }
    if ( ! fprintf(f, "%d\n", pid) ) {
      syslog(LOG_ERR,"start_seti - cant write pid file  %m");
      return;    
    }
  }
}
